@@ -6,7 +6,7 @@ import xml2rfc
 import xml2rfc.utils
 import lxml
 import tempfile
-
+import re
 
 def arrstrip(arr):
     """ Strip beginning and end blanklines of an array """
@@ -31,17 +31,67 @@ def diff_test(case, valid, test, failpath):
 
 class XmlRfcDummy():
     """ Dummy rfc used for test fixtures """
+    def __init__(self):
+        self.pis = {
+            "artworkdelimiter":	None,
+            "artworklines":	0 ,
+            "authorship":	"yes",
+            "autobreaks":	"yes",
+            "background":	"" ,
+            "colonspace":	"no" ,
+            "comments":		"no" ,
+            "docmapping":	"no",
+            "editing":		"no",
+            "emoticonic":	"no",
+            #"footer":		Unset
+            "figurecount":      "no",
+            #"header":		Unset
+            "inline":		"no",
+            "iprnotified":	"no",
+            "linkmailto":	"yes",
+            #"linefile":	Unset
+            #"needLines":       Unset
+            "notedraftinprogress": "yes",
+            "private":		"",
+            "refparent":	"References",
+            "rfcedstyle":	"no",
+            "rfcprocack":	"no",
+            "sectionorphan":    5,
+            "slides":		"no",
+            "sortrefs":		"yes",  # different from default
+            "strict":		"no",
+            "symrefs":		"yes",
+            "tablecount":       "no",
+            "text-list-symbols": "o*+-",
+            "toc":		"no",
+            "tocappendix":	"yes",
+            "tocdepth":		3,
+            "tocindent":	"yes",
+            "tocnarrow":	"yes",
+            "tocompact":	"yes",
+            "topblock":		"yes",
+            #"typeout":		Unset
+            "useobject":	"no" ,
+        }
+        self.pis["compact"] = self.pis["rfcedstyle"]
+        self.pis["subcompact"] = self.pis["compact"]
+
     def getroot(self):
         return lxml.etree.Element('rfc')
 
     def getpis(self):
-        return {
-            'sortrefs': 'yes'
-        }
+        return self.pis
 
     def parse_pi(self, pi):
-        pass
-
+        if pi.text:
+            # Split text in the format 'key="val"'
+            chunks = re.split(r'=[\'"]([^\'"]*)[\'"]', pi.text)
+            # Create pairs from this flat list, discard last element if odd
+            tmp_dict = dict(zip(chunks[::2], chunks[1::2]))
+            for key, val in tmp_dict.items():
+                # Update main PI state
+                self.pis[key] = val
+            # Return the new values added
 
 output_format = [
     {
@@ -51,7 +101,7 @@ output_format = [
         "writer": xml2rfc.PaginatedTextRfcWriter,
     },
     {
-        "ext": "raw",
+        "ext": "raw.txt",
         "spacefix": xml2rfc.utils.formatXmlWhitespace,
         "unicodefix": xml2rfc.utils.safeReplaceUnicode,
         "writer": xml2rfc.RawTextRfcWriter,
@@ -68,6 +118,12 @@ output_format = [
         "unicodefix": lambda x: x,
         "writer": xml2rfc.HtmlRfcWriter,
     },
+    {
+        "ext": "exp.xml",
+        "spacefix": lambda x: x,
+        "unicodefix": lambda x: x,
+        "writer": xml2rfc.HtmlRfcWriter,
+    },
 ]
 
 class WriterElementTest(unittest.TestCase):
@@ -120,10 +176,10 @@ class WriterElementTest(unittest.TestCase):
         return self.function_test("figure_title", "write_t_rec")
 
     def test_texttable_title(self):
-        return self.function_test("texttable_title", "_write_section_rec")
+        return self.function_test("texttable_title", "write_section_rec")
 
     def test_section(self):
-        return self.function_test("section", "_write_section_rec")
+        return self.function_test("section", "write_section_rec")
 
 class WriterRootTest(unittest.TestCase):
     """ Performs tests of full <rfc> + <front> trees against text writer functions """
@@ -161,7 +217,7 @@ class WriterRootTest(unittest.TestCase):
         diff_test(self, self.valid, output, self.failpath)
 
     def status_test(self):
-        self.writer._write_status_section()
+        self.writer.write_status_section()
         output = '\n'.join(arrstrip(self.writer.buf))  # Don't care about initial blank
         diff_test(self, self.valid, output, self.failpath)
 
