@@ -1,10 +1,12 @@
 
 # Static values
 NAME         = 'xml2rfc'
-VERSION      = (2, 3, 11, 1)
+VERSION      = (2, 4, 1, )
 CACHES       = ['/var/cache/xml2rfc', '~/.cache/xml2rfc']  # Ordered by priority
-CACHE_PREFIX = '_HTTP_CACHE'
+CACHE_PREFIX = ''
 NET_SUBDIRS  = ['bibxml', 'bibxml2', 'bibxml3', 'bibxml4', 'bibxml5']
 
 from xml2rfc.parser import *
 from xml2rfc.writers import *
+
+__version__  = '.'.join(map(str, VERSION))
