@@ -13,6 +13,8 @@ import sys
 import parser
 
 quiet = False
+verbose = False
+
 write_out = sys.stdout
 write_err = sys.stderr
 
@@ -22,15 +24,17 @@ def write_on_line(*args):
     write_out.write(' '.join(args))
     write_out.flush()
 
-
 def write(*args):
     """ Prints a message to write_out """
     write_out.write(' '.join(args))
     write_out.write('\n')
 
+def note(*args):
+    if verbose and not quiet:
+        write(*args)
 
 def warn(*args):
-    """ Prints a warning message nuless quiet """
+    """ Prints a warning message unless quiet """
     if not quiet:
         write_err.write('WARNING: ' + ' '.join(args))
         write_err.write('\n')
@@ -47,4 +51,6 @@ def exception(message, list):
         list = [ list ]
     for e in list:
         attr = dict( [ (n,str(getattr(e, n)).replace("\n"," ")) for n in dir(e) if not n.startswith("_") ] )
+        if attr["message"].endswith(", got "):
+            attr["message"] += "nothing."
         write_err.write(" %(filename)s: Line %(line)s: %(message)s\n" % attr )
