@@ -10,6 +10,10 @@ import os
 import shutil
 import xml2rfc.log
 from urlparse import urlparse, urljoin
+try:
+    import debug
+except ImportError:
+    pass
 
 __all__ = ['XmlRfcParser', 'XmlRfc', 'XmlRfcError']
 
@@ -32,7 +36,8 @@ class CachingResolver(lxml.etree.Resolver):
     """ Custom ENTITY request handler that uses a local cache """
     def __init__(self, cache_path=None, library_dirs=None, source=None,
                  templates_path='templates', verbose=False, quiet=False,
-                 network_loc='http://xml.resource.org/public/rfc/'):
+                 network_loc='http://xml.resource.org/public/rfc/',
+                 rfc_number=None):
         self.verbose = verbose
         self.quiet = quiet
         self.source = source
@@ -40,6 +45,7 @@ class CachingResolver(lxml.etree.Resolver):
         self.templates_path = templates_path
         self.network_loc = network_loc
         self.include = False
+        self.rfc_number = rfc_number
 
         # Get directory of source
         self.source_dir = os.path.abspath(os.path.dirname(self.source))
@@ -59,8 +65,7 @@ class CachingResolver(lxml.etree.Resolver):
             else:
                 try:
                     os.makedirs(dir)
-                    if self.verbose:
-                        xml2rfc.log.write('Created cache directory at', dir)
+                    xml2rfc.log.note('Created cache directory at', dir)
                     self.write_cache = dir
                 except OSError:
                     # Can't write to this directory, try the next one
@@ -94,6 +99,9 @@ class CachingResolver(lxml.etree.Resolver):
         # If the source itself is requested, return as-is
         if request == self.source:
             return self.resolve_filename(request, context)
+        if request == u"internal:/rfc.number":
+            if self.rfc_number:
+                return self.resolve_string(self.rfc_number, context)
         if not urlparse(request).netloc:
             # Format the request from the relative path of the source so that 
             # We get the exact same path as in the XML document
@@ -238,10 +246,10 @@ class CachingResolver(lxml.etree.Resolver):
                              '\n    '.join(attempts)
             raise error
         else:
-            if not cached and self.verbose:
+            if not cached:
                 # Haven't printed a verbose messsage yet
                 typename = self.include and 'include' or 'entity'
-                xml2rfc.log.write('Resolving ' + typename + '...', result)
+                xml2rfc.log.note('Resolving ' + typename + '...', result)
             return result
 
     def cache(self, url):
@@ -255,9 +263,8 @@ class CachingResolver(lxml.etree.Resolver):
         for dir in self.read_caches:
             cached_path = os.path.join(dir, xml2rfc.CACHE_PREFIX, basename)
             if os.path.exists(cached_path):
-                if self.verbose:
-                    xml2rfc.log.write('Resolving ' + typename + '...', url)
-                    xml2rfc.log.write('Loaded from cache', cached_path)
+                xml2rfc.log.note('Resolving ' + typename + '...', url)
+                xml2rfc.log.note('Loaded from cache', cached_path)
                 return cached_path
         # Not found, save to `write_cache`
         if self.write_cache:
@@ -265,12 +272,12 @@ class CachingResolver(lxml.etree.Resolver):
                                       xml2rfc.CACHE_PREFIX, basename)
             try:
                 xml2rfc.utils.StrictUrlOpener().retrieve(url, write_path)
-                if self.verbose:
-                    xml2rfc.log.write('Resolving ' + typename + '...', url)
-                    xml2rfc.log.write('Created cache at', write_path)
+                xml2rfc.log.note('Resolving ' + typename + '...', url)
+                xml2rfc.log.note('Created cache at', write_path)
                 return write_path
-            except IOError:
+            except IOError, e:
                 # Invalid URL -- Error will be displayed in getReferenceRequest
+                xml2rfc.log.note("I/O Error: %s" % e)
                 return ''
         # No write cache available, test existance of URL and return
         else:
@@ -280,17 +287,21 @@ class CachingResolver(lxml.etree.Resolver):
                 return url
             except IOError:
                 # Invalid URL
+                xml2rfc.log.note("I/O Error: %s" % e)
                 return ''
 
 class XmlRfcParser:
-    """ XML parser with callbacks to construct an RFC tree """
+
+    """ XML parser container with callbacks to construct an RFC tree """
     def __init__(self, source, verbose=False, quiet=False,
                  cache_path=None, templates_path=None, library_dirs=None,
                  network_loc='http://xml.resource.org/public/rfc/'):
         self.verbose = verbose
         self.quiet = quiet
         self.source = source
-            
+        self.cache_path = cache_path
+        self.network_loc = network_loc
+
         # Initialize templates directory
         self.templates_path = templates_path or \
                               os.path.join(os.path.dirname(xml2rfc.__file__),
@@ -314,24 +325,56 @@ class XmlRfcParser:
             if raw_dir not in self.library_dirs:
                 self.library_dirs.append(raw_dir)
 
-
-        # Initialize the caching system
-        self.cachingResolver = CachingResolver(cache_path=cache_path,
+        # Initialize the caching system.  We'll replace this later if parsing.
+        self.cachingResolver = CachingResolver(cache_path=self.cache_path,
                                         library_dirs=self.library_dirs,
                                         templates_path=self.templates_path,
-                                        source=source,
-                                        network_loc=network_loc,
-                                        verbose=verbose, quiet=quiet)
-        
+                                        source=self.source,
+                                        network_loc=self.network_loc,
+                                        verbose=self.verbose,
+                                        quiet=self.quiet,
+                                    )
+
     def delete_cache(self, path=None):
         self.cachingResolver.delete_cache(path=path)
 
-    def parse(self, remove_comments=True, remove_pis=False):
+    def parse(self, remove_comments=True, remove_pis=False, quiet=False):
         """ Parses the source XML file and returns an XmlRfc instance """
-        if not self.quiet:
+        if not (self.quiet or quiet):
             xml2rfc.log.write('Parsing file', self.source)
 
-        # Get a parser object
+        # Get an iterating parser object
+        context = lxml.etree.iterparse(self.source,
+                                      dtd_validation=False,
+                                      load_dtd=True,
+                                      attribute_defaults=True,
+                                      no_network=False,
+                                      remove_comments=remove_comments,
+                                      remove_pis=remove_pis,
+                                      remove_blank_text=True,
+                                      resolve_entities=False,
+                                      events=("start",),
+                                      tag="rfc",
+                                  )
+        # resolver without knowledge of rfc_number:
+        caching_resolver = CachingResolver(cache_path=self.cache_path,
+                                        library_dirs=self.library_dirs,
+                                        templates_path=self.templates_path,
+                                        source=self.source,
+                                        network_loc=self.network_loc,
+                                        verbose=self.verbose,
+                                        quiet=self.quiet,
+                                    )
+        context.resolvers.add(caching_resolver)
+
+        # Get hold of the rfc number (if any) in the rfc element, so we can
+        # later resolve the "&rfc.number;" entity.
+        for action, element in context:
+            if element.tag == "rfc":
+                self.rfc_number = element.attrib.get("number", None)
+                break
+
+        # now get a regular parser, and parse again, this time resolving entities
         parser = lxml.etree.XMLParser(dtd_validation=False,
                                       load_dtd=True,
                                       attribute_defaults=True,
@@ -341,6 +384,17 @@ class XmlRfcParser:
                                       remove_blank_text=True,
                                       resolve_entities=True)
 
+        # Initialize the caching system
+        self.cachingResolver = CachingResolver(cache_path=self.cache_path,
+                                        library_dirs=self.library_dirs,
+                                        templates_path=self.templates_path,
+                                        source=self.source,
+                                        network_loc=self.network_loc,
+                                        verbose=self.verbose,
+                                        quiet=self.quiet,
+                                        rfc_number = self.rfc_number,
+                                    )
+
         # Add our custom resolver
         parser.resolvers.add(self.cachingResolver)
 
@@ -390,7 +444,51 @@ class XmlRfc:
     def __init__(self, tree, default_dtd_path):
         self.default_dtd_path = default_dtd_path
         self.tree = tree
-        self.pis = {}
+        # Pi default values
+        self.pis = {
+            "artworkdelimiter":	None,
+            "artworklines":	0 ,
+            "authorship":	"yes",
+            "autobreaks":	"yes",
+            "background":	"" ,
+            "colonspace":	"no" ,
+            "comments":		"no" ,
+            "docmapping":	"no",
+            "editing":		"no",
+            "emoticonic":	"no",
+            #"footer":		Unset
+            "figurecount":      "no",
+            #"header":		Unset
+            "inline":		"no",
+            "iprnotified":	"no",
+            "linkmailto":	"yes",
+            #"linefile":	Unset
+            #"needLines":       Unset
+            "notedraftinprogress": "yes",
+            "private":		"",
+            "refparent":	"References",
+            "rfcedstyle":	"no",
+            "rfcprocack":	"no",
+            "sectionorphan":    5,
+            "slides":		"no",
+            "sortrefs":		"no",
+            "strict":		"no",
+            "symrefs":		"yes",
+            "tablecount":       "no",
+            "text-list-symbols": "o*+-",
+            "toc":		"no",
+            "tocappendix":	"yes",
+            "tocdepth":		3,
+            "tocindent":	"yes",
+            "tocnarrow":	"yes",
+            "tocompact":	"yes",
+            "topblock":		"yes",
+            #"typeout":		Unset
+            "useobject":	"no" ,
+        }
+        # Special cases:
+        self.pis["compact"] = self.pis["rfcedstyle"]
+        self.pis["subcompact"] = self.pis["compact"]
 
     def getroot(self):
         """ Wrapper method to get the root of the XML tree"""
@@ -420,7 +518,7 @@ class XmlRfc:
                 xml2rfc.log.error('DTD file does not exist:', dtd_path)
                 return False, []
             
-        # Otherwise, use documents DTD declaration
+        # Otherwise, use document's DTD declaration
         else:
             dtd = self.tree.docinfo.externalDTD
             
@@ -433,8 +531,11 @@ class XmlRfc:
             # The document was valid
             return True, []
         else:
-            # The document was not valid
-            return False, dtd.error_log
+            if len(dtd.error_log) == 0:
+                return True, []
+            else:
+                # The document was not valid
+                return False, dtd.error_log
     
     def parse_pi(self, pi):
         """ Add a processing instruction to the current state 
@@ -462,7 +563,6 @@ class XmlRfc:
         """
         # Grab processing instructions from xml tree
         element = self.tree.getroot().getprevious()
-        pairs = []
         while element is not None:
             if element.tag is lxml.etree.PI:
                 self.parse_pi(element)
