@@ -8,6 +8,10 @@ import re
 import textwrap
 import string
 import urllib
+try:
+    import debug
+except ImportError:
+    pass
 
 import xml2rfc.log
 
@@ -37,12 +41,11 @@ class MyTextWrapper(textwrap.TextWrapper):
         self.wordsep_re_uni = re.compile(self.wordsep_re.pattern, re.U)
 
         # Override end of line regex, double space after '].'
-        self.sentence_end_re = \
-                  re.compile(r'([a-z0-9\]")]'  # lowercase, digit, bracket, parens, quote
-                             r'|^[A-Z]{2,})'   # acronyms         
-                             r'[\.\!\?]'       # sentence-ending punct.
-                             r'[\"\']?'        # optional end-of-quote
-                             r'\Z')            # end of chunk
+        self.sentence_end_re = re.compile(r'[%s%s0-9\>\]\)\"\']'  # lowercase letter, end parentheses
+                            r'[\.\!\?]'     # sentence-ending punct.
+                            r'[\"\'\)\]]?'  # optional end-of-quote, end parentheses
+                            r'\Z'           # end of chunk
+                            % (string.lowercase, string.uppercase))
 
     def replace(self, text):
         """ Replace control entities with the proper character 
@@ -52,7 +55,8 @@ class MyTextWrapper(textwrap.TextWrapper):
             text = re.sub(re.escape(key), val, text)
         return text
 
-    def wrap(self, text, break_on_hyphens=True, initial_indent='', subsequent_indent=''):
+    def wrap(self, text, break_on_hyphens=True, initial_indent='', subsequent_indent='',
+        fix_doublespace=True, fix_sentence_endings=True):
         """ Mirrored implementation of wrap which replaces characters properly
             also lets you easily specify indentation on the fly
         """
@@ -60,15 +64,33 @@ class MyTextWrapper(textwrap.TextWrapper):
         self.initial_indent = initial_indent
         self.subsequent_indent = subsequent_indent
         self.break_on_hyphens = break_on_hyphens
+        self.fix_sentence_endings = fix_sentence_endings
 
         # Original implementation
         text = self._munge_whitespace(text)
 
-        # Replace some characters after splitting has occured
-        chunks = map(self.replace, self._split(text))
+        # Maybe remove double (and more) spaces, except when they might be between sentences
+        if fix_doublespace:
+            text = re.sub("([^].!?])  +", r"\1 ", text)
+            text = re.sub("([.!?])   +", r"\1  ", text)
 
+        # prevent breaking "Section N.N" and "Appendix X.X"
+        text = re.sub("(Section|Appendix) ", r"\1&#160;", text)
+
+        # Replace some characters after splitting has occured
+        parts = map(self.replace, self._split(text))
+        chunks = []
+        max_word_len = self.width - len(subsequent_indent)
+        for chunk in parts:
+            if len(chunk) > max_word_len:
+                bits = self._split(chunk)
+                chunks += bits
+            else:
+                chunks += [ chunk ]
+        
         # Original implementation
         if self.fix_sentence_endings:
+            pre = chunks
             self._fix_sentence_endings(chunks)
         return self._wrap_chunks(chunks)
 
@@ -122,7 +144,38 @@ def formatXmlWhitespace(tree):
                                element.tail))
 
 
-def int2roman(number, uppercase=False):
+# ----------------------------------------------------------------------
+# Base conversions.
+# From http://tech.vaultize.com/2011/08/python-patterns-number-to-base-x-and-the-other-way/
+
+DEFAULT_DIGITS = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ'
+ 
+def num_to_baseX(num, digits=DEFAULT_DIGITS):
+   if num < 0: return '-' + num_to_baseX(-num)
+   if num == 0: return digits[0]
+   X = len(digits)
+   s = ''
+   while num > 0:
+        s = digits[num % X] + s
+        num //= X
+ 
+   return s
+ 
+def baseX_to_num(s, digits=DEFAULT_DIGITS):
+   if s[0] == '-': return -1 * baseX_to_num(s[1:])
+   ctopos = dict([(c, pos) for pos, c in enumerate(digits)])
+   X = len(digits)
+   num = 0
+   for c in s: num = num * X + ctopos[c]
+   return num
+
+# ----------------------------------------------------------------------
+# Use the generic base conversion to create list letters
+
+def int2letter(num):
+    return num_to_baseX(num, "abcdefghijklmnopqrstuvwxyz")
+
+def int2roman(number):
     numerals = { 
         1 : "i", 
         4 : "iv", 
@@ -143,7 +196,7 @@ def int2roman(number, uppercase=False):
         while number >= value:
             result += numeral
             number -= value
-    return uppercase and result.upper() or result
+    return result
 
 
 def urlkeep(text):
@@ -371,3 +424,13 @@ _post_break_replacements = {
     '&#8209;': '-',  # nbhy
     '&#8288;': '',   # wj
 }
+
+def get_initials(author):
+    """author is an rfc2629 author element.  Return the author initials,
+    fixed up according to current flavour and policy."""
+    initials = author.attrib.get('initials', '').split()
+    for i in range(len(initials)):
+        initial = initials[i]
+        if not initial.endswith('.'):
+            initials[i] = initial+'.'
+    return ' '.join(initials)
