@@ -9,8 +9,10 @@ import datetime
 import lxml
 import xml2rfc.log
 import xml2rfc.utils
-from xml2rfc.utils import urlkeep
-
+try:
+    import debug
+except ImportError:
+    pass
 
 class _RfcItem:
     """ A unique ID object for an anchored RFC element.
@@ -73,7 +75,6 @@ class BaseRfcWriter:
         'table_align':              'center',
         'table_style':              'full',
         'ttcol_align':              'left',
-        'references_title':         'References',
         'ipr':                      'trust200902',
         'submissionType':           'IETF',
         'consensus':                'no',
@@ -172,6 +173,11 @@ class BaseRfcWriter:
         'This document is a product of the Internet Architecture Board ' \
         '(IAB) and represents information that the IAB has deemed valuable ' \
         'to provide for permanent record.'
+    boilerplate['status']['IAB_consensus'] = (
+        'This document is a product of the Internet Architecture Board ' 
+        '(IAB) and represents information that the IAB has deemed valuable '
+        'to provide for permanent record.  It represents the consensus of '
+        'the Internet Architecture Board (IAB).')
     boilerplate['status']['independent'] = \
         'This is a contribution to the RFC Series, independently of any ' \
         'other RFC stream.  The RFC Editor has chosen to publish this ' \
@@ -193,22 +199,22 @@ class BaseRfcWriter:
         '5741.'
 
     # Paragraph 3
-    boilerplate['status']['p3'] = urlkeep(''
+    boilerplate['status']['p3'] = (
         'Information about the current status of this document, any errata, '
         'and how to provide feedback on it may be obtained at '
         'http://www.rfc-editor.org/info/rfc%s.')
 
     # 'Status of this Memo' boilerplate for drafts
-    boilerplate['status']['draft'] = map(urlkeep,
-       ['Internet-Drafts are working documents of the Internet Engineering '
+    boilerplate['status']['draft'] = [
+       'Internet-Drafts are working documents of the Internet Engineering '
        'Task Force (IETF).  Note that other groups may also distribute '
-       'working documents as Internet-Drafts.  The list of current Internet- '
+       'working documents as Internet-Drafts.  The list of current Internet-'
        'Drafts is at http://datatracker.ietf.org/drafts/current/.',
-
+       #
        'Internet-Drafts are draft documents valid for a maximum of six months '
        'and may be updated, replaced, or obsoleted by other documents at any '
        'time.  It is inappropriate to use Internet-Drafts as reference '
-       'material or to cite them other than as "work in progress."'])
+       'material or to cite them other than as "work in progress."']
     boilerplate['draft_expire'] = \
        'This Internet-Draft will expire on %s.'
 
@@ -224,7 +230,7 @@ class BaseRfcWriter:
     boilerplate['base_copyright_header'] = \
         'Copyright (c) %s IETF Trust and the persons identified as the ' \
         'document authors.  All rights reserved.'
-    boilerplate['base_copyright_body'] = urlkeep(''
+    boilerplate['base_copyright_body'] = (
         'This document is subject to BCP 78 and the IETF Trust\'s Legal '
         'Provisions Relating to IETF Documents '
         '(http://trustee.ietf.org/license-info) in effect on the date of '
@@ -407,7 +413,7 @@ class BaseRfcWriter:
             month = date.attrib.get('month', '')
             # If year is this year, and month not specified, use current date
             if not year or (year == str(today.year) and not month) or \
-                           (year == str(today.year) and month == today.strftime("%B")):
+                           (self.draft and year == str(today.year) and month == today.strftime("%B")):
                 # Set everything to today
                 date.attrib['year'] = today.strftime('%Y')
                 date.attrib['month'] = today.strftime('%B')
@@ -432,7 +438,7 @@ class BaseRfcWriter:
                     # Warn about no date
                     xml2rfc.log.warn('No date specified for document.')
 
-    def _format_counter(self, text, count):
+    def _format_counter(self, text, count, list_length=1):
         """ Return a proper string for a formatted list bullet.  Allowed types:
                 %c: Lowercase chars
                 %C: Uppercase chars
@@ -440,12 +446,26 @@ class BaseRfcWriter:
                 %i: Lowercase roman numerals
                 %I: Uppercase roman numerals
         """
-        return (text
-            .replace(r'%d', str(count))
-            .replace(r'%c', str(string.ascii_lowercase[(count - 1) % 26]))
-            .replace(r'%C', str(string.ascii_uppercase[(count - 1) % 26]))
-            .replace(r'%i', xml2rfc.utils.int2roman(count).ljust(5)) 
-            .replace(r'%I', xml2rfc.utils.int2roman(count, uppercase=True).ljust(5)))
+        import math
+        roman_widths = {        1:1,  2:2,  3:3,  4:2,  5:1,  6:2,  7:3,  8:4,  9:2,
+                        10:1, 11:2, 12:3, 13:4, 14:3, 15:2, 16:3, 17:4, 18:5, 19:3,
+                        20:2, 21:3, 22:4, 23:5, 24:4, 25:3, 26:4, 27:5, 28:6, 29:4, }
+        #
+        decimal_width = int(math.log(list_length, 10))
+        roman_width = roman_widths.get(list_length, 6)
+        letter_width = int(math.log(list_length, 26))
+        extra_width = len(text)+1
+        if '%d' in text:
+            text = text.replace(r'%d', str(count)).ljust(decimal_width+extra_width)
+        elif '%c' in text:
+            text = text.replace(r'%c', xml2rfc.utils.int2letter(count-1)).ljust(letter_width+extra_width)
+        elif '%C' in text:
+            text = text.replace(r'%C', xml2rfc.utils.int2letter(count-1).upper()).ljust(letter_width+extra_width)
+        elif '%i' in text:
+            text = text.replace(r'%i', xml2rfc.utils.int2roman(count)).ljust(roman_width+extra_width)
+        elif '%I' in text:
+            text = text.replace(r'%I', xml2rfc.utils.int2roman(count).upper()).ljust(roman_width+extra_width)
+        return text
 
     def _format_author_string(self, authors):
         """ Given a list of <author> elements, return a readable string of names """
@@ -454,10 +474,7 @@ class BaseRfcWriter:
             organization = author.find('organization')
             surname = author.attrib.get('surname', '')
             if surname:
-                initials = author.attrib.get('initials', '')
-                # Append a dot if it doesnt already exist
-                if initials and not initials.endswith('.'):
-                    initials = initials + '.'
+                initials = xml2rfc.utils.get_initials(author)
                 if i == len(authors) - 1 and len(authors) > 1:
                     # Last author is rendered in reverse
                     buf.append('and ' + initials + ' ' + \
@@ -540,7 +557,7 @@ class BaseRfcWriter:
         """ Returns a list of lines for the top right header """
         lines = []
         # Render author?
-        authorship = self.pis.get('authorship', 'yes')
+        authorship = self.pis['authorship']
         if authorship == 'yes':
             # Keep track of previous organization and remove if redundant.
             last_org = None
@@ -548,10 +565,7 @@ class BaseRfcWriter:
                 role = author.attrib.get('role', '')
                 if role == 'editor':
                     role = ', Ed.'
-                initials = author.attrib.get('initials', '')
-                # Append a dot if it doesnt already exist
-                if initials and not initials.endswith('.'):
-                    initials = initials + '.'
+                initials = xml2rfc.utils.get_initials(author)
                 lines.append(initials + ' ' + author.attrib.\
                              get('surname', '') + role)
                 organization = author.find('organization')
@@ -583,14 +597,14 @@ class BaseRfcWriter:
             lines = map(string.strip, lines)
         return lines
 
-    def _write_figure(self, figure):
+    def write_figure(self, figure):
         """ Writes <figure> elements """
         align = figure.attrib.get('align', self.defaults['figure_align'])
         anchor = figure.attrib.get('anchor')
         title = figure.attrib.get('title', self.defaults['figure_title'])
 
         # Keep track of count if there is an anchor, or PI was enabled
-        if anchor or self.pis.get('figurecount', 'no') == 'yes':
+        if anchor or self.pis['figurecount'] == 'yes':
             self.figure_count += 1
         
         if anchor:
@@ -607,12 +621,12 @@ class BaseRfcWriter:
             self.write_t_rec(preamble, align=align)
 
         # Write figure with optional delimiter
-        delimiter = self.pis.get('artworkdelimiter', None)
+        delimiter = self.pis['artworkdelimiter']
         artwork = figure.find('artwork')
         # artwork_align = artwork.attrib.get('align', align)
         # Explicitly use figure alignment
         artwork_align = align
-        blanklines = int(self.pis.get('artworklines', 0))
+        blanklines = int(self.pis['artworklines'])
         self.write_raw(figure.find('artwork').text, align=artwork_align,
                        blanklines=blanklines, delimiter=delimiter,
                        source_line=figure.sourceline)
@@ -624,7 +638,7 @@ class BaseRfcWriter:
 
         # Write label
         title = figure.attrib.get('title', '')
-        if anchor or self.pis.get('figurecount', 'no') == 'yes':
+        if anchor or self.pis['figurecount'] == 'yes':
             if title:
                 title = ': ' + title
             self.write_label('Figure ' + str(self.figure_count) + title,
@@ -632,14 +646,14 @@ class BaseRfcWriter:
         else:
             self.write_label(title, type='figure', source_line=figure.sourceline)
 
-    def _write_table(self, table):
+    def write_table(self, table):
         """ Writes <texttable> elements """
         align = table.attrib.get('align', self.defaults['table_align'])
         anchor = table.attrib.get('anchor')
         title = table.attrib.get('title', self.defaults['table_title'])
 
         # Keep track of count if there is an anchor, or PI was enabled
-        if anchor or self.pis.get('tablecount', 'no') == 'yes':
+        if anchor or self.pis['tablecount'] == 'yes':
             self.table_count += 1
 
         if anchor:
@@ -664,7 +678,7 @@ class BaseRfcWriter:
             self.write_t_rec(postamble, align=align)
 
         # Write label if anchor is set or PI figurecount = yes
-        if anchor or self.pis.get('tablecount', 'no') == 'yes':
+        if anchor or self.pis['tablecount'] == 'yes':
             title = table.attrib.get('title', '')
             if title:
                 title = ': ' + title
@@ -678,7 +692,7 @@ class BaseRfcWriter:
         pass
         
 
-    def _write_section_rec(self, section, count_str="1.", appendix=False, \
+    def write_section_rec(self, section, count_str="1.", appendix=False, \
                            level=0):
         """ Recursively writes <section> elements 
         
@@ -690,8 +704,7 @@ class BaseRfcWriter:
             anchor = section.attrib.get('anchor')
             title = section.attrib.get('title')
             include_toc = section.attrib.get('toc', self.defaults['section_toc']) != 'exclude' \
-                          and (not appendix or self.pis.get('tocappendix', \
-                                                            'yes') == 'yes')
+                          and (not appendix or self.pis['tocappendix'] == 'yes')
             if self.indexmode:
                 # Add section to the index
                 self._indexSection(count_str, title=title, anchor=anchor,
@@ -722,9 +735,9 @@ class BaseRfcWriter:
                 self.write_t_rec(element, autoAnchor=autoAnchor)
                 p_count += 1
             elif element.tag == 'figure':
-                self._write_figure(element)
+                self.write_figure(element)
             elif element.tag == 'texttable':
-                self._write_table(element)
+                self.write_table(element)
 
         s_count = 1  # Section counter
         
@@ -736,11 +749,11 @@ class BaseRfcWriter:
         for child_sec in section.findall('section'):
             if appendix == True and not count_str:
                 # Use an alphabetic counter for first-level appendix
-                self._write_section_rec(child_sec, string.uppercase[s_count - 1],
+                self.write_section_rec(child_sec, string.uppercase[s_count - 1],
                                         level=level + 1, appendix=True)
             else:
                 # Use a numeric counter
-                self._write_section_rec(child_sec, count_str + str(s_count), 
+                self.write_section_rec(child_sec, count_str + str(s_count), 
                                         level=level + 1, appendix=appendix)
 
             s_count += 1
@@ -749,7 +762,7 @@ class BaseRfcWriter:
         if count_str == '' and appendix == False:
             self.ref_start = s_count
 
-    def _write_status_section(self):
+    def write_status_section(self):
         """ Writes the 'Status of this Memo' section """
 
         self.write_heading('Status of This Memo', autoAnchor='rfc.status')
@@ -788,6 +801,11 @@ class BaseRfcWriter:
                     else:
                         p2.append(self.boilerplate['status']['IRTF_workgroup'] \
                                   % workgroup)
+            elif stream == 'IAB':
+                if consensus == 'yes':
+                    p2.append(self.boilerplate['status']['IAB_consensus'])
+                else:
+                    p2.append(self.boilerplate['status']['IAB'])
             else:
                 p2.append(self.boilerplate['status'].get(stream, ''))
 
@@ -825,7 +843,7 @@ class BaseRfcWriter:
                 self.write_paragraph( \
                     self.boilerplate['draft_expire'] % self.expire_string)
     
-    def _write_copyright(self):
+    def write_copyright(self):
         """ Writes the 'Copyright' section """
         self.write_heading('Copyright Notice', autoAnchor='rfc.copyrightnotice')
 
@@ -862,14 +880,14 @@ class BaseRfcWriter:
         # Middle sections
         middle = self.r.find('middle')
         if middle is not None:
-            self._write_section_rec(middle, None)
+            self.write_section_rec(middle, None)
 
         # References sections
         # Treat references as nested only if there is more than one
         ref_counter = str(self.ref_start)
         references = self.r.findall('back/references')
         # Write root level references header
-        ref_title = self.pis.get('refparent', self.defaults['references_title'])
+        ref_title = self.pis['refparent']
         if len(references) == 1:
             ref_title = references[0].attrib.get('title', ref_title)
 
@@ -878,15 +896,14 @@ class BaseRfcWriter:
         if len(references) > 1:
             for i, reference_list in enumerate(references):
                 ref_newcounter = ref_counter + '.' + str(i + 1)
-                ref_title = reference_list.attrib.get('title',
-                                        self.defaults['references_title'])
+                ref_title = reference_list.attrib.get('title', self.pis["refparent"])
                 self._indexReferences(ref_newcounter, title=ref_title, \
                                       subCounter=i+1, level=2)
 
         # Appendix sections
         back = self.r.find('back')
         if back is not None:
-            self._write_section_rec(back, None, appendix=True)
+            self.write_section_rec(back, None, appendix=True)
 
         # Index section, disable if there are no irefs
         if len(self._iref_index) > 0:
@@ -915,7 +932,7 @@ class BaseRfcWriter:
         self.table_count = 0
 
         # Block header
-        topblock = self.pis.get('topblock', 'yes')
+        topblock = self.pis['topblock']
         if topblock == 'yes':
             self.write_top(self._prepare_top_left(), \
                                self._prepare_top_right())
@@ -934,7 +951,7 @@ class BaseRfcWriter:
                 self.write_t_rec(t)
 
         # Optional notified boilerplate
-        if self.pis.get('iprnotified', 'no') == 'yes':
+        if self.pis['iprnotified'] == 'yes':
             self.write_paragraph(BaseRfcWriter.boilerplate['iprnotified'])
 
         # Optional notes
@@ -947,27 +964,27 @@ class BaseRfcWriter:
         self._validate_ipr()
 
         # "Status of this Memo" section
-        self._write_status_section()
+        self.write_status_section()
 
         # Copyright section
-        self._write_copyright()
+        self.write_copyright()
 
         # Insert the table of contents marker at this position
-        toc_enabled = self.pis.get('toc', 'no')
+        toc_enabled = self.pis['toc']
         if toc_enabled == 'yes':
             self.insert_toc()
 
         # Middle sections
         middle = self.r.find('middle')
         if middle is not None:
-            self._write_section_rec(middle, None)
+            self.write_section_rec(middle, None)
 
         # References sections
         # Treat references as nested only if there is more than one
         ref_counter = str(self.ref_start)
         references = self.r.findall('back/references')
         # Write root level references header
-        ref_title = self.pis.get('refparent', self.defaults['references_title'])
+        ref_title = self.pis['refparent']
         if len(references) == 1:
             ref_title = references[0].attrib.get('title', ref_title)
 
@@ -976,8 +993,7 @@ class BaseRfcWriter:
         if len(references) > 1:
             for i, reference_list in enumerate(references):
                 ref_newcounter = ref_counter + '.' + str(i + 1)
-                ref_title = reference_list.attrib.get('title',
-                                        self.defaults['references_title'])
+                ref_title = reference_list.attrib.get('title', self.pis['refparent'])
                 autoAnchor = 'rfc.references.' + str(i + 1)
                 self.write_heading(ref_title, bullet=ref_newcounter + '.',\
                                    autoAnchor=autoAnchor, level=2)
@@ -988,7 +1004,7 @@ class BaseRfcWriter:
         # Appendix sections
         back = self.r.find('back')
         if back is not None:
-            self._write_section_rec(back, None, appendix=True)
+            self.write_section_rec(back, None, appendix=True)
 
         # Index section, disable if there are no irefs
         if len(self._iref_index) > 0:
