@@ -4,7 +4,7 @@
 
 import lxml.etree
 import xml2rfc
-
+import datetime
 
 class ExpandedXmlWriter:
     """ Writes a duplicate XML file but with all references expanded """
@@ -12,7 +12,7 @@ class ExpandedXmlWriter:
     # Note -- we don't need to subclass BaseRfcWriter because the behavior
     # is so different and so trivial
 
-    def __init__(self, xmlrfc, quiet=False, verbose=False):
+    def __init__(self, xmlrfc, quiet=False, verbose=False, date=datetime.date.today()):
         self.root = xmlrfc.getroot()
         self.quiet = quiet
         self.verbose = verbose
