@@ -10,6 +10,10 @@ import string
 import sys
 import datetime
 import cgi
+try:
+    import debug
+except ImportError:
+    pass
 
 # Local libs
 import xml2rfc
@@ -62,7 +66,7 @@ class HtmlRfcWriter(BaseRfcWriter):
         # Table to insert the iref into
         self.iref_table = None
 
-    def _write_list(self, list, parent):
+    def write_list(self, list, parent):
         style = list.attrib.get('style', 'empty')
         if style == 'hanging':
             list_elem = E.DL()
@@ -119,24 +123,31 @@ class HtmlRfcWriter(BaseRfcWriter):
         # Add link for toc itself
         link = E.LINK(rel='Contents', href='#rfc.toc')
         self.buffers['toc_head_links'].append(self._serialize(link))
+        tocdepth = self.pis['tocdepth']
+        try:
+            tocdepth = int(tocdepth)
+        except ValueError:
+            xml2rfc.log.warn('Invalid tocdepth specified, must be integer:', \
+                             tocdepth)
+            tocdepth = 3
         for item in self._getTocIndex():
-            # Create link for head
-            link = E.LINK(href='#' + item.autoAnchor)
-            link.attrib['rel'] = 'copyright' in item.autoAnchor and \
-                                 'Copyright' or 'Chapter'
-            if item.title and item.counter:
-                link.attrib['title'] = item.counter + ' ' + item.title
-            self.buffers['toc_head_links'].append(self._serialize(link))
-            # Create actual toc list item
-            a = E.A(item.title, href='#' + item.autoAnchor)
-            counter_text = item.counter and item.counter + '.   ' or ''
-            # Prepend appendix at first level
-            if item.level == 1 and item.appendix:
-                counter_text = "Appendix " + counter_text
-            li = E.LI(counter_text)
-            li.append(a)
-            self.buffers['toc_rows'].append(self._serialize(li))
-
+            if item.level <= tocdepth:
+                # Create link for head
+                link = E.LINK(href='#' + item.autoAnchor)
+                link.attrib['rel'] = 'copyright' in item.autoAnchor and \
+                                     'Copyright' or 'Chapter'
+                if item.title and item.counter:
+                    link.attrib['title'] = item.counter + ' ' + item.title
+                self.buffers['toc_head_links'].append(self._serialize(link))
+                # Create actual toc list item
+                a = E.A(item.title, href='#' + item.autoAnchor)
+                counter_text = item.counter and item.counter + '.   ' or ''
+                # Prepend appendix at first level
+                if item.level == 1 and item.appendix:
+                    counter_text = "Appendix " + counter_text
+                li = E.LI(counter_text)
+                li.append(a)
+                self.buffers['toc_rows'].append(self._serialize(li))
     def _serialize(self, element):
         return lxml.html.tostring(element, pretty_print=True, method='xml')
     
@@ -253,11 +264,13 @@ class HtmlRfcWriter(BaseRfcWriter):
                       level=1):
         # Use a hierarchy of header tags if docmapping set
         h = E.H1()
-        if self.pis.get('docmapping', 'no') == 'yes':
-            if level > 1:
+        if self.pis['docmapping'] == 'yes':
+            if level == 2:
                 h = E.H2()
-            elif level > 2:
+            elif level == 3:
                 h = E.H3()
+            elif level >= 4:
+                h = E.H4()
         if autoAnchor:
             h.attrib['id'] = autoAnchor
         if bullet:
@@ -307,7 +320,12 @@ class HtmlRfcWriter(BaseRfcWriter):
         else:
             current = parent
         if t.text:
-            current.text = t.text
+            if "anchor" in t.attrib:
+                a = E.A(name=t.attrib["anchor"])
+                a.tail = t.text
+                current.append(a)
+            else:
+                current.text = t.text
             if autoAnchor:
                 current.attrib['id'] = autoAnchor
         for child in t:
@@ -325,15 +343,15 @@ class HtmlRfcWriter(BaseRfcWriter):
                 if child.tail:
                     br.tail = child.tail
             elif child.tag == 'list':
-                self._write_list(child, parent)
+                self.write_list(child, parent)
                 if child.tail:
                     parent.append(E.P(child.tail))
             elif child.tag == 'figure':
                 # Callback to base writer method
-                self._write_figure(child)
+                self.write_figure(child)
             elif child.tag == 'texttable':
                 # Callback to base writer method
-                self._write_table(child)
+                self.write_table(child)
         # If we are back at top level, serialize the whole temporary structure
         # Add to body buffer
         if parent == self.temp_div:
@@ -416,7 +434,7 @@ class HtmlRfcWriter(BaseRfcWriter):
             if email is not None and email.text:
                 span = E.SPAN('EMail: ')
                 span.attrib['class'] = 'vcardline'
-                if self.pis.get('linkmailto', 'yes') == 'yes':
+                if self.pis['linkmailto'] == 'yes':
                     span.append(E.A(email.text, href='mailto:' + email.text))
                 else:
                     span.text += email.text
@@ -440,7 +458,7 @@ class HtmlRfcWriter(BaseRfcWriter):
         for i, reference in enumerate(list.findall('reference')):
             tr = E.TR()
             # Use anchor or num depending on PI
-            if self.pis.get('symrefs', 'yes') == 'yes':
+            if self.pis['symrefs'] == 'yes':
                 bullet = reference.attrib.get('anchor', str(i + 1))
             else:
                 bullet = str(i + 1)
@@ -454,7 +472,7 @@ class HtmlRfcWriter(BaseRfcWriter):
                 organization = author.find('organization')
                 email = author.find('address/email')
                 surname = author.attrib.get('surname')
-                initials = author.attrib.get('initials', '')
+                initials = xml2rfc.utils.get_initials(author)
                 if surname is not None:
                     if j == len(authors) - 1 and len(authors) > 1:
                         # Last author, render in reverse
@@ -464,7 +482,7 @@ class HtmlRfcWriter(BaseRfcWriter):
                         name_string = surname + ', ' + initials
                     a = E.A(name_string)
                     if email is not None and email.text:
-                        if self.pis.get('linkmailto', 'yes') == 'yes':
+                        if self.pis['linkmailto'] == 'yes':
                             a.attrib['href'] = 'mailto:' + email.text
                     if organization is not None and organization.text:
                         a.attrib['title'] = organization.text
@@ -528,7 +546,7 @@ class HtmlRfcWriter(BaseRfcWriter):
         # Add caption, if it exists
         if 'title' in table.attrib and table.attrib['title']:
             caption = ''
-            if table_num and self.pis.get('tablecount', 'no') == 'yes':
+            if table_num and self.pis['tablecount'] == 'yes':
                 caption = 'Table ' + str(table_num) + ': '
             htmltable.append(E.CAPTION(caption + table.attrib['title']))
 
@@ -631,8 +649,8 @@ class HtmlRfcWriter(BaseRfcWriter):
         self._create_toc()
         
         # Grab values that haven't been inserted yet
-        background_image = self.pis.get('background', '') and \
-            "background-image:url('%s');" % self.pis.get('background') or ''
+        background_image = (self.pis['background'] and 
+            "background-image:url('%s');" % self.pis['background'] or '')
         title = self.r.find('front/title').text
         docName = self.r.attrib.get('docName', '')
         description = ''
