@@ -4,6 +4,10 @@
 
 # Python libs
 import datetime
+try:
+    import debug
+except ImportError:
+    pass
 
 # Local libs
 from xml2rfc.writers.base import BaseRfcWriter
@@ -44,6 +48,20 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
     # We'll store each marking as a dictionary of line_num: section_length.
     # This way we can step through these markings during writing to
     # preemptively construct appropriate page breaks.
+    def write_figure(self, *args, **kwargs):
+        """ Override base writer to add a marking """
+        begin = len(self.buf)
+        BaseRfcWriter.write_figure(self, *args, **kwargs)
+        end = len(self.buf)
+        self.break_hints[begin] = (end - begin, "txt")
+
+    def write_table(self, *args, **kwargs):
+        """ Override base writer to add a marking """
+        begin = len(self.buf)
+        BaseRfcWriter.write_table(self, *args, **kwargs)
+        end = len(self.buf)
+        self.break_hints[begin] = (end - begin, "txt")
+
     def write_raw(self, *args, **kwargs):
         """ Override text writer to add a marking """
         begin = len(self.buf)
@@ -51,10 +69,10 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
         end = len(self.buf)
         self.break_hints[begin] = (end - begin, "raw")
 
-    def _write_text(self, *args, **kwargs):
+    def write_text(self, *args, **kwargs):
         """ Override text writer to add a marking """
         begin = len(self.buf)
-        RawTextRfcWriter._write_text(self, *args, **kwargs)
+        RawTextRfcWriter.write_text(self, *args, **kwargs)
         end = len(self.buf)
         self.break_hints[begin] = (end - begin, "txt")
         
@@ -63,10 +81,10 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
         self.break_hints[len(self.buf)] = (0, "break")
         
     def _toc_size_hint(self):
-        return len(self._write_toc(paging=True))
+        return len(self.write_toc(paging=True))
     
     def _iref_size_hint(self):
-        return len(self._write_iref_index())
+        return len(self.write_iref_index())
     
     # ------------------------------------------------------------------------
     
@@ -74,11 +92,16 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
                       level=1):
         # Store the line number of this heading with its unique anchor, 
         # to later create paging info
-        line_num = len(self.buf)
-        self.heading_marks[line_num] = autoAnchor
+        begin = len(self.buf)
+        self.heading_marks[begin] = autoAnchor
         RawTextRfcWriter.write_heading(self, text, bullet=bullet, \
                                        autoAnchor=autoAnchor, anchor=anchor, \
                                        level=level)
+        # Reserve room for a blankline and some lines of section content
+        end = len(self.buf) + self.pis["sectionorphan"]
+                                        # text, in order to prevent orphan headings
+        self.break_hints[begin] = (end - begin, "txt")
+                                        
 
     def pre_indexing(self):
         pass
@@ -203,7 +226,7 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
 
                 if (text_type == "break"
                     or (text_type == "raw" and needed > available and needed < max_page_length-2 )
-                    or (self.pis.get('autobreaks', 'yes') == 'yes'
+                    or (self.pis['autobreaks'] == 'yes'
                         and needed > available
                         and needed < max_page_length-2
                         and (needed-available < 2 or available < 2) ) ):
@@ -249,7 +272,7 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
         # Now we need to go back into the buffer and insert the real table 
         # of contents and iref based on the pointers we created
         if len(toc_pointers) > 0:
-            tocbuf = self._write_toc(paging=True)
+            tocbuf = self.write_toc(paging=True)
             ptr, end = toc_pointers.pop(0)
             for line in tocbuf:
                 self.output[ptr] = line
@@ -261,7 +284,7 @@ class PaginatedTextRfcWriter(RawTextRfcWriter):
                         break
 
         if len(iref_pointers) > 0:
-            irefbuf = self._write_iref_index()
+            irefbuf = self.write_iref_index()
             ptr, end = iref_pointers.pop(0)
             for line in irefbuf:
                 self.output[ptr] = line
