@@ -9,6 +9,11 @@ import string
 import math
 import lxml
 import datetime
+import re
+try:
+    import debug
+except ImportError:
+    pass
 
 # Local lib
 from xml2rfc.writers.base import BaseRfcWriter
@@ -39,7 +44,7 @@ class RawTextRfcWriter(BaseRfcWriter):
         self.iref_marks = {}
 
         # Text lookups
-        self.list_symbols = self.pis.get('text-list-symbols', 'o*+-')
+        self.list_symbols = self.pis['text-list-symbols']
         self.inline_tags = ['xref', 'eref', 'iref', 'cref', 'spanx']
         
         # Custom textwrapper object
@@ -60,9 +65,10 @@ class RawTextRfcWriter(BaseRfcWriter):
         """ <vspace> line break wrapper to allow for overrides """
         return self._lb(num=num)
 
-    def _write_text(self, string, indent=0, sub_indent=0, bullet='',
+    def write_text(self, string, indent=0, sub_indent=0, bullet='',
                   align='left', leading_blankline=False, buf=None,
-                  strip=True, edit=False, source_line=None):
+                  strip=True, edit=False, wrap_urls=True,
+                  fix_sentence_endings=True, source_line=None):
         """ Writes a line or multiple lines of text to the buffer.
 
             Several parameters are included here.  All of the API calls
@@ -72,7 +78,7 @@ class RawTextRfcWriter(BaseRfcWriter):
         """
         if not buf:
             buf = self.buf
-        # We can take advantage of textwraps initial_indent by using a bullet
+        # We can take advantage of textwrap's initial_indent by using a bullet
         # parameter and treating it separately.  We still need to indent it.
         subsequent = ' ' * (indent + sub_indent)
         if bullet:
@@ -85,7 +91,7 @@ class RawTextRfcWriter(BaseRfcWriter):
             initial = subsequent
 
         if leading_blankline:
-            if edit and self.pis.get('editing', 'no') == 'yes':
+            if edit and self.pis['editing'] == 'yes':
                 # Render an editing mark
                 self.edit_counter += 1
                 self._lb(buf=buf, text=str('<' + str(self.edit_counter) + '>'))
@@ -95,9 +101,12 @@ class RawTextRfcWriter(BaseRfcWriter):
             if strip:
                 # Strip initial whitespace
                 string = string.lstrip()
-            par = self.wrapper.wrap(string,
-                                    initial_indent=initial,
-                                    subsequent_indent=subsequent)
+            if wrap_urls:
+                par = self.wrapper.wrap(string, initial_indent=initial, subsequent_indent=subsequent,
+                                        fix_sentence_endings=fix_sentence_endings)
+            else:
+                par = self.wrapper.wrap(xml2rfc.utils.urlkeep(string), initial_indent=initial,
+                                        subsequent_indent=subsequent, fix_sentence_endings=fix_sentence_endings)                    
             if align == 'left':
                 buf.extend(par)
             elif align == 'center':
@@ -116,17 +125,18 @@ class RawTextRfcWriter(BaseRfcWriter):
             # print the bullet
             buf.append(initial)
 
-    def _write_list(self, list, level=0, indent=3):
+    def write_list(self, list, level=0, indent=3):
         """ Writes a <list> element """
         bullet = '   '
         hangIndent = None
         style = list.attrib.get('style', 'empty')
+        listlength = len(list.findall('t'))        
         if style == 'hanging' or style.startswith('format'):
             # Check for optional hangIndent
             try:
-                hangIndent = int(list.attrib.get('hangIndent', 3))
+                hangIndent = int(list.attrib.get('hangIndent', 3+level*3))
             except (ValueError, TypeError):
-                hangIndent = 3
+                hangIndent = 3 + level*3
         format_str = None
         counter_index = None
         if style.startswith('format'):
@@ -151,17 +161,15 @@ class RawTextRfcWriter(BaseRfcWriter):
             elif element.tag == 't':
                 # Disable linebreak if subcompact=yes AND not first list element
                 leading_blankline = True
-                if t_count > 0 and self.pis.get('subcompact', \
-                    self.pis.get('compact', \
-                    self.pis.get('rfcedstyle', 'no'))) == 'yes':
+                if t_count > 0 and self.pis['subcompact'] == 'yes':
                     leading_blankline = False
                 if style == 'symbols':
                     bullet = self.list_symbols[level % len(self.list_symbols)]
                     bullet += '  '
                 elif style == 'numbers':
-                    bullet = str(t_count + 1) + '.  '
+                    bullet = self._format_counter("%d.", t_count+1, listlength)
                 elif style == 'letters':
-                    bullet = string.ascii_lowercase[t_count % 26] + '.  '
+                    bullet = self._format_counter("%c.", t_count+1, listlength)
                 elif style == 'hanging':
                     bullet = element.attrib.get('hangText', '')
                     if len(bullet) < hangIndent:
@@ -169,15 +177,26 @@ class RawTextRfcWriter(BaseRfcWriter):
                         bullet = bullet.ljust(hangIndent)
                     else:
                         # Insert a single space
-                        bullet += ' '
+                        bullet += '  '
                     # Add an extra space in front of colon if colonspace enabled
                     if bullet.endswith(':') and \
-                    self.pis.get('colonspace', 'no') == 'yes':
+                    self.pis['colonspace'] == 'yes':
                         bullet+= ' '
+                    if element.text and len(bullet) > self.width/2:
+                        # extra check of remaining space if the bullet is
+                        # very long
+                        first_word = self.wrapper._split(element.text)[0]
+                        if len(first_word) > (self.width - len(bullet) - indent):
+                            self.write_text('', bullet=bullet, indent=indent,
+                                leading_blankline=leading_blankline)
+                            self._vspace()
+                            leading_blankline=False
+                            indent = hangIndent
+                            bullet = ''
                 elif style.startswith('format'):
                     self.list_counters[counter_index] += 1
                     count = self.list_counters[counter_index]
-                    bullet = self._format_counter(format_str, count) + ' '
+                    bullet = self._format_counter(format_str, count, listlength)
                 if hangIndent:
                     sub_indent = hangIndent
                 else:
@@ -188,7 +207,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                 t_count += 1
 
         
-    def _write_toc(self, paging=False):
+    def write_toc(self, paging=False):
         """ Write table of contents to a temporary buffer and return """
         if self.toc_marker < 1:
             # Toc is either disabled, or the pointer was messed up
@@ -196,7 +215,7 @@ class RawTextRfcWriter(BaseRfcWriter):
         tmpbuf = ['', 'Table of Contents', '']
         # Retrieve toc from the index
         tocindex = self._getTocIndex()
-        tocdepth = self.pis.get('tocdepth', '3')
+        tocdepth = self.pis['tocdepth']
         try:
             tocdepth = int(tocdepth)
         except ValueError:
@@ -204,7 +223,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                              tocdepth)
             tocdepth = 3
         indent_scale = 2
-        if self.pis.get('tocnarrow', 'yes') == 'no':
+        if self.pis['tocnarrow'] == 'no':
             indent_scale = 3
         for item in tocindex:
             # Add decoration to counter if it exists, otherwise leave empty
@@ -217,7 +236,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                         counter += ' '
                 # Get item depth based on its section 'level' attribute
                 depth = item.level - 1
-                if depth < 0 or self.pis.get('tocindent', 'yes') == 'no':
+                if depth < 0 or self.pis['tocindent'] == 'no':
                     depth = 0
                 # Prepend appendix at first level
                 if item.level == 1 and item.appendix:
@@ -240,7 +259,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                 tmpbuf.extend(lines)
         return tmpbuf
             
-    def _write_iref_index(self):
+    def write_iref_index(self):
         """ Write iref index to a temporary buffer and return """
         if self.iref_marker < 1:
             # iref is either disabled, or the pointer was messed up
@@ -256,16 +275,16 @@ class RawTextRfcWriter(BaseRfcWriter):
                 alpha_bucket[letter] = [key]
         for letter in sorted(alpha_bucket.keys()):
             # Write letter
-            self._write_text(letter, indent=3, leading_blankline=True, buf=tmpbuf)
+            self.write_text(letter, indent=3, leading_blankline=True, buf=tmpbuf)
             for item in alpha_bucket[letter]:
                 pages = self._iref_index[item].pages
                 # Write item
-                self._write_text(item + '  ' + ', '.join(map(str, pages))
+                self.write_text(item + '  ' + ', '.join(map(str, pages))
                                                         , indent=6, buf=tmpbuf)
                 for subitem in self._iref_index[item].subitems:
                     pages = self._iref_index[item].subitems[subitem].pages
                     # Write subitem
-                    self._write_text(subitem + '  ' + ', '.join(map(str,pages))
+                    self.write_text(subitem + '  ' + ', '.join(map(str,pages))
                                                         , indent=9, buf=tmpbuf)
         return tmpbuf
 
@@ -282,6 +301,7 @@ class RawTextRfcWriter(BaseRfcWriter):
             target_text = item.title
         else: #Default
             target_text = item.autoName
+        target_text = re.sub("([./-])", r"\1&#8288;", target_text)   # word joiner, to prevent line breaks
         if xref.text:
             if not target_text.startswith('['):
                 target_text = '(' + target_text + ')'
@@ -289,17 +309,17 @@ class RawTextRfcWriter(BaseRfcWriter):
         else:
             return target_text
 
-    def _write_ref_element(self, key, text, sub_indent, source_line=None):
+    def write_ref_element(self, key, text, sub_indent, source_line=None):
         """ Render a single reference element """
         # Use an empty first line if key is too long
         min_spacing = 2
         if len(key) + min_spacing > sub_indent:
-            self._write_text(key, indent=3, leading_blankline=True, source_line=source_line)
-            self._write_text(text, indent=3 + sub_indent, source_line=source_line)
+            self.write_text(key, indent=3, leading_blankline=True, wrap_urls=False, fix_sentence_endings=False, source_line=source_line)
+            self.write_text(text, indent=3 + sub_indent, wrap_urls=False, fix_sentence_endings=False, source_line=source_line)
         else:
             # Fill space to sub_indent in the bullet
-            self._write_text(text, indent=3, bullet=key.ljust(sub_indent), \
-                     sub_indent=sub_indent, leading_blankline=True, source_line=source_line)
+            self.write_text(text, indent=3, bullet=key.ljust(sub_indent), \
+                     sub_indent=sub_indent, leading_blankline=True, wrap_urls=False, fix_sentence_endings=False, source_line=source_line)
     
     def _combine_inline_elements(self, elements):
         """ Shared function for <t> and <c> elements
@@ -348,7 +368,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                         self.iref_marks[pos] = []
                     self.iref_marks[pos].append((item, subitem))
             elif element.tag == 'cref' and \
-                self.pis.get('comments', 'no') == 'yes':                
+                self.pis['comments'] == 'yes':                
                 # Render if processing instruction is enabled
                 anchor = element.attrib.get('anchor', '')
                 if anchor:
@@ -387,7 +407,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                 xml2rfc.log.warn("Output line (from source around line %s) is %s characters; longer than %s.  Excess characters: '%s':\n  '%s'\n"
                     % (source_line+num, len(line), self.width, line[self.width:], line))
             else:
-                xml2rfc.log.warn("Output line is %s characters; longer than %s.  Excess characters: '%s':\n  '%s'\n"
+                xml2rfc.log.warn("Output line (from source around line %s) is %s characters; longer than %s.  Excess characters: '%s':\n  '%s'\n"
                     % (buf_line+num, len(line), self.width, line[self.width:], line))
 
     # ---------------------------------------------------------
@@ -405,6 +425,7 @@ class RawTextRfcWriter(BaseRfcWriter):
     def write_raw(self, text, indent=3, align='left', blanklines=0, \
                   delimiter=None, leading_blankline=True, source_line=None):
         """ Writes a raw stream of characters, preserving space and breaks """
+        
         if text:
             if leading_blankline:
                 # Start with a newline
@@ -427,7 +448,7 @@ class RawTextRfcWriter(BaseRfcWriter):
             if align == 'center':
                 # Find the longest line, and use that as a fixed center.
                 longest_line = len(max(lines, key=len))
-                center_indent = ((self.width - longest_line) / 2)
+                center_indent = indent + ((self.width - indent - longest_line) / 2)
                 indent_str = center_indent > indent and ' ' * center_indent or \
                                                         ' ' * indent
                 for line in lines:
@@ -452,24 +473,25 @@ class RawTextRfcWriter(BaseRfcWriter):
 
     def write_label(self, text, type='figure', source_line=None):
         """ Writes a centered label """
-        self._write_text(text, indent=3, align='center', leading_blankline=True, source_line=source_line)
+        self.write_text(text, indent=3, align='center', leading_blankline=True, source_line=source_line)
 
     def write_title(self, title, docName=None, source_line=None):
         """ Write the document title and (optional) name """
-        self._write_text(title, leading_blankline=True, align='center', source_line=source_line)
+        self.write_text(title, leading_blankline=True, align='center', source_line=source_line)
         if docName is not None:
-            self._write_text(docName, align='center')
+            self.write_text(docName, align='center')
 
     def write_heading(self, text, bullet='', autoAnchor=None, anchor=None, \
                       level=1):
         """ Write a generic header """
         if bullet:
             bullet += '  '
-        self._write_text(text, bullet=bullet, indent=0, leading_blankline=True)
+        self.write_text(text, bullet=bullet, indent=0, leading_blankline=True)
 
     def write_paragraph(self, text, align='left', autoAnchor=None):
         """ Write a generic paragraph of text.  Used for boilerplate. """
-        self._write_text(text, indent=3, align=align, leading_blankline=True)
+        text = xml2rfc.utils.urlkeep(text)
+        self.write_text(text, indent=3, align=align, leading_blankline=True)
 
     def write_t_rec(self, t, indent=3, sub_indent=0, bullet='',
                      autoAnchor=None, align='left', level=0, leading_blankline=True):
@@ -485,7 +507,7 @@ class RawTextRfcWriter(BaseRfcWriter):
             current_text += inline_text
             if (current_text and not current_text.isspace()) or bullet:
                 # Attempt to write a paragraph of inline text
-                self._write_text(current_text, indent=indent,
+                self.write_text(current_text, indent=indent,
                                 leading_blankline=leading_blankline,
                                 sub_indent=sub_indent, bullet=bullet,
                                 edit=True, align=align, source_line=t.sourceline)
@@ -503,13 +525,13 @@ class RawTextRfcWriter(BaseRfcWriter):
                     else:
                         new_indent = len(bullet) + indent
                     # Call sibling function to construct list
-                    self._write_list(element, indent=new_indent, level=level)
+                    self.write_list(element, indent=new_indent, level=level)
                     # Auto-break for tail paragraph
                     leading_blankline = True
                     bullet = ''
 
                 elif element.tag == 'figure':
-                    self._write_figure(element)
+                    self.write_figure(element)
                     # Auto-break for tail paragraph
                     leading_blankline = True
                     bullet = ''
@@ -596,7 +618,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                 lines.append('Fax:   ' + facsimile.text)
             email = address.find('email')
             if email is not None and email.text:
-                label = self.pis.get('rfcedstyle', 'no') == 'yes' and 'EMail' or 'Email'
+                label = self.pis['rfcedstyle'] == 'yes' and 'EMail' or 'Email'
                 lines.append('%s: %s' % (label, email.text))
             uri = address.find('uri')
             if uri is not None and uri.text:
@@ -642,7 +664,7 @@ class RawTextRfcWriter(BaseRfcWriter):
             refstring.append('.')
             annotation = ref.find('annotation')
             # Use anchor or num depending on PI
-            if self.pis.get('symrefs', 'yes') == 'yes':
+            if self.pis['symrefs'] == 'yes':
                 bullet = '[' + ref.attrib.get('anchor', str(i + 1)) + ']'
             else:
                 bullet = '[' + str(i + 1) + ']'
@@ -653,15 +675,15 @@ class RawTextRfcWriter(BaseRfcWriter):
             if annotation is not None and annotation.text:
                 # Render annotation as a separate paragraph
                 annotationdict[bullet] = annotation.text
-        if self.pis.get('sortrefs', 'no') == 'yes':
+        if self.pis['sortrefs'] == 'yes':
             refkeys = sorted(refkeys)
         # Hard coded indentation amount
         refindent = 11
         for key in refkeys:
-            self._write_ref_element(key, refdict[key], refindent, source_line=refsource[key])
+            self.write_ref_element(key, refdict[key], refindent, source_line=refsource[key])
             # Render annotation as a separate paragraph
             if key in annotationdict:
-                self._write_text(annotationdict[key], indent=refindent + 3,
+                self.write_text(annotationdict[key], indent=refindent + 3,
                                  leading_blankline=True, source_line=refsource[key])
 
     def draw_table(self, table, table_num=None):
@@ -682,6 +704,11 @@ class RawTextRfcWriter(BaseRfcWriter):
         num_columns = len(matrix[0])
         for i, cell in enumerate(table.findall('c')):
             if i % num_columns == 0:
+                # Insert blank row if PI 'compact' is 'no'
+                if self.pis["compact"] == "no" and row > 0:
+                    row += 1
+                    matrix.append(['']*num_columns)
+                    pass
                 row += 1
                 matrix.append([])
             text = cell.text or ''
@@ -690,6 +717,7 @@ class RawTextRfcWriter(BaseRfcWriter):
                 inline_text, null = \
                     self._combine_inline_elements(cell.getchildren())
                 text += inline_text
+            text = self.wrapper.replace(text)
             matrix[row].append(text)
 
         # Get table style and determine maximum width of table
@@ -775,10 +803,8 @@ class RawTextRfcWriter(BaseRfcWriter):
         
         # Now construct the cells using textwrap against column_widths
         cell_lines = [
-            [
-                textwrap.wrap(cell, column_widths[j]) \
-                for j, cell in enumerate(matrix[i])
-            ] for i in range(0, len(matrix))
+            [ textwrap.wrap(cell, column_widths[j]) or [''] for j, cell in enumerate(matrix[i]) ]
+            for i in range(0, len(matrix))
         ]
 
         output = []
@@ -799,7 +825,9 @@ class RawTextRfcWriter(BaseRfcWriter):
 
         # Draw the table
         for i, cell_line in enumerate(cell_lines):
-            for row in range(max(map(len, cell_line))):
+            # produce as many outpur rows as the number of wrapped
+            # text lines in the cell with most lines, but at least 1
+            for row in range(0, max(map(len, cell_line))):
                 if style == 'headers' or style == 'none':
                     line = ['']
                 else:
@@ -858,9 +886,9 @@ class RawTextRfcWriter(BaseRfcWriter):
     def post_rendering(self):
         # Insert the TOC and IREF into the main buffer
         self.output = self.buf[:self.toc_marker] + \
-                      self._write_toc() + \
+                      self.write_toc() + \
                       self.buf[self.toc_marker:self.iref_marker] + \
-                      self._write_iref_index() + \
+                      self.write_iref_index() + \
                       self.buf[self.iref_marker:]
 
     def write_to_file(self, file):
